#!/usr/bin/python
# -*- coding: UTF-8 -*-
import os
from niprov.filesystem import Filesystem
from niprov.commandline import Commandline
from niprov.filefilter import FileFilter
from niprov.jsonfile import JsonFile
from niprov.files import FileFactory


def discover(root, filefilter=FileFilter(), filesys=Filesystem(), 
        listener=Commandline(), repository=JsonFile(), file=FileFactory()):
    """
    Search a directory for image files, inspect them and store provenance.

    Files are only included if they match the filters in the 
    'discovery-filter.txt' file.
    If a file is already known, it will be ignored and the listener informed.

    Args:
        root (str): The top directory in which to look for new files.
    """
    dirs = filesys.walk(root)
    for (root, sdirs, files) in dirs:
        for filename in files:
            filepath = os.path.join(root, filename)
            if filefilter.include(filename):
                img = file.locatedAt(filepath)
                if repository.knows(img):
                    listener.knownFile(img.path)
                elif repository.knowsSeries(img):
                    series = repository.getSeries(img)
                    series.addFile(img)
                    repository.update(series)
                    listener.fileFoundInSeries(img, series)
                else:
                    try:
                        img.inspect()
                    except:
                        listener.fileError(img.path)
                    else:
                        repository.add(img.provenance)
                        listener.fileFound(img)
       



#!/usr/bin/python
# -*- coding: UTF-8 -*-
from niprov.jsonfile import JsonFile
from niprov.exporters import ExportFactory
from niprov.commandline import Commandline


def report(format=None, forFile=None, forSubject=None, 
        repository=JsonFile(), exportFactory=ExportFactory(), 
        listener=Commandline()):
    """Publish or simply return provenance for selected files.

    To get provenance on one specific file, pass its path as the 'forFile' 
    argument. Alternatively, to get all files associated with a certain subject,
    use the 'forSubject' argument. If none of these is used, provenance for all
    files is reported.

    Args:
        format (str): The format in which to publish the provenance. 
            Currently only 'html'.
        forFile (str): Select one file based on this path.
        forSubject (str): Select files regarding this subject.

    Returns:
        Provenance reported. Either a list of dicts, or a dict.
    """
    exporter = exportFactory.createExporter(format)
    if forFile:
        if not repository.knowsByPath(forFile):
            listener.unknownFile(forFile)
            return
        provenance = repository.byPath(forFile)
        exporter.export(provenance)
    elif forSubject:
        provenance = repository.bySubject(forSubject)
        exporter.exportList(provenance)
    else:
        provenance = repository.all()
        exporter.exportList(provenance)
    return provenance



#!/usr/bin/python
# -*- coding: UTF-8 -*-
from niprov.jsonfile import JsonFile


def add(new, transient=False, repository=JsonFile()):
    """
    Simply register the file.

    This does not add any significant provenance for this file, but makes it 
    known to the provenance data, such that image files can be logged that have 
    been created using this file. Useful i.e. for temporary files.

    Args:
        new (str): Path to the newly created file.
        transient (bool, optional): Set this to True to indicate that the file 
            is only temporary and future checks should not expect it to be 
            physically present. Defaults to False, assuming that the file 
            remains.

    Returns:
        dict: New provenance
    """
    provenance = {}
    provenance['path'] = new
    provenance['transient'] = transient
    repository.add(provenance)
    return provenance



    



import unittest
from mock import Mock

class ExportFactoryTests(unittest.TestCase):

    def test_Html(self):
        from niprov.exporters import ExportFactory
        from niprov.html import HtmlExporter
        exporter = ExportFactory().createExporter('html')
        self.assertIsInstance(exporter, HtmlExporter)

    def test_Default(self):
        from niprov.exporters import ExportFactory, DummyExporter
        exporter = ExportFactory().createExporter(None)
        self.assertIsInstance(exporter, DummyExporter)







import unittest
import os, shutil


class ApiTests(unittest.TestCase):

    def setUp(self):
        self.dbpath = os.path.expanduser(os.path.join('~','provenance.json'))
        if os.path.exists(self.dbpath):
            shutil.move(self.dbpath, self.dbpath.replace('.json','.backup.json'))

    def tearDown(self):
        if os.path.exists(self.dbpath):
            shutil.move(self.dbpath, self.dbpath.replace('.json','.test.json'))

    def test_Discover(self):
        import niprov
        niprov.discover('testdata')
        provenance = niprov.report(forFile='testdata/dicom/T1.dcm')
        self.assertEqual(provenance['dimensions'], [80, 80, 10])
        provenance = niprov.report(forFile='testdata/eeg/stub.cnt')
        self.assertEqual(provenance['subject'], 'Jane Doe')

    def test_Rename(self):
        try:
            import niprov
            files, directory = self.createExtensionlessFiles()
            niprov.renameDicoms(directory)
            assert not os.path.isfile(files[0])
            assert os.path.isfile(files[0]+'.dcm')
        finally:
            self.clearDicomfiles()

    def createExtensionlessFiles(self):
        if not os.path.exists('dicomdir'):
            os.makedirs('dicomdir')
        files = []
        for n in range(5):
            f = os.path.join('dicomdir',str(n))
            files.append(f)
            with open(f,'w') as fhandle:
                fhandle.write('x')
        return files, 'dicomdir'

    def clearDicomfiles(self):
        shutil.rmtree('dicomdir')





import unittest
import mock
from mock import Mock
import os.path as ospath

class DiscoveryTests(unittest.TestCase):

    def setUp(self):
        self.repo = Mock()
        self.repo.knows.return_value = False
        self.repo.knowsSeries.return_value = False
        self.img1 = Mock()
        self.img2 = Mock()
        self.img3 = Mock()
        self.fileFactory = Mock()
        self.fileFactory.locatedAt.side_effect = [self.img1, self.img2, self.img3]
        self.filesys = Mock()
        self.filesys.walk.return_value = [('root',[],['known','new','unknown'])]
        self.listener = Mock()
        self.filt = Mock()
        self.inspect = Mock()
        self.inspect.side_effect = lambda x: ('p', x)

    def discover(self, path):
        import niprov.discovery
        niprov.discovery.inspect = self.inspect
        niprov.discovery.discover(path, filesys=self.filesys, listener=self.listener,
            filefilter=self.filt, repository=self.repo, file=self.fileFactory)

    def test_Tells_listener_about_files_found(self):
        self.setupFilter('.x')
        self.filesys.walk.return_value = [('root',[],['/p/f1.x','/p/f2.x']),
            ('root',[],['/p/p2/f3.x'])] #(dirpath, dirnames, filenames)
        self.discover('root')
        self.filesys.walk.assert_called_with('root')
        self.listener.fileFound.assert_any_call(self.img1)
        self.listener.fileFound.assert_any_call(self.img2)
        self.listener.fileFound.assert_any_call(self.img3)

    def test_file_filters(self):
        self.setupFilter('valid.file')
        self.filesys.walk.return_value = [('root',[],['valid.file','other.file'])]
        self.discover('root')
        self.listener.fileFound.assert_any_call(self.img1)

    def test_Creates_ImageFile_object_with_factory(self):
        self.setupFilter('.valid')
        self.filesys.walk.return_value = [('root',[],['a.valid', 'other.file'])]
        self.discover('root')
        self.fileFactory.locatedAt.assert_any_call('root/a.valid')
        self.assertRaises(AssertionError,
            self.fileFactory.locatedAt.assert_any_call,'root/other.file')

    def test_Calls_inspect(self):
        self.setupFilter('.valid')
        self.filesys.walk.return_value = [('root',[],['a.valid','b.valid'])]
        self.discover('root')
        self.img1.inspect.assert_called_with()

    def test_Hands_provenance_to_repository(self):
        self.setupFilter('.valid')
        self.filesys.walk.return_value = [('root',[],['a.valid','other.file','b.valid'])]
        self.discover('root')
        self.repo.add.assert_any_call(self.img1.provenance)
        self.repo.add.assert_any_call(self.img2.provenance)

    def test_If_discovers_file_that_is_known_ignore_it(self):
        self.repo.knows.side_effect = lambda f: True if f==self.img2 else False
        self.discover('root')
        self.repo.add.assert_any_call(self.img1.provenance)
        self.assertNotCalledWith(self.repo.add, self.img2.provenance)
        self.listener.knownFile.assert_called_with(self.img2.path)

    def test_If_repo_doesnt_know_file_but_knows_series_update_series(self):
        self.repo.knows.return_value = False
        self.repo.knowsSeries.side_effect = lambda f: True if f==self.img2 else False
        series = Mock()
        self.repo.getSeries.return_value = series
        self.discover('root')
        series.addFile.assert_called_with(self.img2)
        self.repo.update.assert_called_with(series)
        self.listener.fileFoundInSeries.assert_called_with(self.img2, series)

    def test_If_inspect_raises_exceptions_tells_listener_and_doesnt_save(self):
        self.img1.inspect.side_effect = IOError
        self.discover('root')
        self.assertNotCalledWith(self.repo.add, self.img1.provenance)
        self.listener.fileError.assert_called_with(self.img1.path)

    def assertNotCalledWith(self, m, *args, **kwargs):
        c = mock.call(*args, **kwargs)
        assert c not in m.call_args_list, "Unexpectedly found call: "+str(c)

    def setupFilter(self, valid):
        def filter_side_effect(*args):
            if valid in args[0]:
                return True
            return False
        self.filt.include = Mock(side_effect=filter_side_effect)
        




#!/usr/bin/python
# -*- coding: UTF-8 -*-
from niprov.filesystem import Filesystem
from niprov.commandline import Commandline
from niprov.html import HtmlExporter
from niprov.externals import Externals


class ExportFactory(object):

    def __init__(self):
        self.listener = Commandline()
        self.filesys = Filesystem()

    def createExporter(self, format):
        if format is None:
            return DummyExporter()
        elif format == 'html':
            return HtmlExporter(self.filesys, self.listener, Externals())
        else:
            raise ValueError('Unknown format: '+str(format))


class DummyExporter(object):

    def __init__(self, **kwargs):
        pass

    def exportList(self, provenance):
        pass

    def export(self, provenance):
        pass
